def task_status_to_str(stat: int):
  """
  将任务状态转换为字符串

  :param stat: 任务状态
  :return: 转换后的字符串
  """
  log_str = ''
  if (stat & int('1000000000000000', 2)) == 0:
    log_str += '未发现银联小票，请检查银联小票名字是否命名为“pos小票”；'
  
  if (stat & int('0100000000000000', 2)) == 0:
    log_str += '未发现宜家小票，请检查宜家小票名字是否命名为“小票”；'

  if (stat & int('0010000000000000', 2)) == 0:
    log_str += '未发现发票，请检查发票名字是否命名为“发票”；'

  if (stat & int('0001000000000000', 2)) == 0:
    log_str += '未发现送货单，请检查送货单名字是否命名为“清单”；'
  
  if (stat & int('0000100000000000', 2)) == 0:
    log_str += '未发现商品图片，请检查和银联小票认真对比参考号；'

  if (stat & int('0000010000000000', 2)) == 0:
    log_str += '未开始识别银联小票，请耐心等待；'

  # if (stat & int('0000001000000000', 2)) == 0:
  #   log_str += '未开始识别宜家小票，请耐心等待；'

  if (stat & int('0000000100000000', 2)) == 0:
    log_str += '未开始识别发票，请耐心等待；'

  if (stat & int('0000000010000000', 2)) == 0:
    log_str += '未开始识别送货单，请耐心等待；'

  return log_str


